package no.ntnu.idatt1002.demo.data.recipes;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;

import static org.junit.jupiter.api.Assertions.*;

class FileHandlerTest {

    RecipeRegister recipeRegister = new RecipeRegister();
    IngredientsAtHand ingredientsAtHand = new IngredientsAtHand();

    Recipe recipe1;
    Recipe recipe2;

    @BeforeEach
    void beforeEach() {
        recipe1 = new Recipe("Meat, cheese and potatoes", "Instructions");

        recipe1.addIngredient(FoodItem.MINCED_MEAT, 500, MeasuringUnit.GR);
        recipe1.addIngredient(FoodItem.POTATO, 750, MeasuringUnit.GR);
        recipe1.addIngredient(FoodItem.YELLOW_CHEESE, 2, MeasuringUnit.DL);

        recipe2 = new Recipe("Meat, cheese and lemons", "Instructions");

        recipe2.addIngredient(FoodItem.MINCED_MEAT, 500, MeasuringUnit.GR);
        recipe2.addIngredient(FoodItem.LEMON, 750, MeasuringUnit.GR);
        recipe2.addIngredient(FoodItem.YELLOW_CHEESE, 2, MeasuringUnit.DL);

        recipeRegister.addRecipe(recipe1);
        recipeRegister.addRecipe(recipe2);

        // For AtHand:

        ingredientsAtHand.addIngredient(FoodItem.POTATO);
        ingredientsAtHand.addIngredient(FoodItem.MILK);
        ingredientsAtHand.addIngredient(FoodItem.LEMON);
        ingredientsAtHand.addIngredient(FoodItem.MINCED_MEAT);


    }


    @Test
    @DisplayName("Write recipe register correctly to file as text.")
    void writeRecipeRegisterToFile() throws FileNotFoundException {
        assertAll(() -> FileHandler.writeRegister(recipeRegister, "RecipeRegister"));
        //FileHandler.readToTerminal("RecipeRegister");
    }

    @Test
    @DisplayName("Read recipe register correctly from file to object.")
    void readRecipeRegisterFromFile() throws FileNotFoundException {
        // Read test register into a register object.
        // Write the object and print to terminal.

        assertAll(() -> FileHandler.readRecipeRegister("testReadRegisterFromFile"));

        RecipeRegister registerReadFromTestFile = FileHandler.readRecipeRegister("testReadRegisterFromFile");

        // Write the register object to file and read to terminal:
        FileHandler.writeRegister(registerReadFromTestFile, "ReadAndWrittenRegister");

        //FileHandler.readToTerminal("ReadAndWrittenRegister");

        // Another iteration to spot accumulating spaces or new lines.
        RecipeRegister secondRegister = FileHandler.readRecipeRegister("ReadAndWrittenRegister");
        FileHandler.writeRegister(secondRegister, "secondGeneration");

    }


    // Ingredients at hand nesting


    @Test
    @DisplayName("Write ingredients at hand to file.")
    void writeIngredientsAtHandToFile() throws FileNotFoundException {
        assertAll(() -> FileHandler.writeIngredientsAtHand(ingredientsAtHand, "AtHandRegister"));
        //FileHandler.readToTerminal("AtHandRegister");
    }


}